"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeServerRouteWrapper = void 0;
const logger_1 = require("@coder/logger");
const express = __importStar(require("express"));
const util_1 = require("../../common/util");
const constants_1 = require("../constants");
const cli_1 = require("../cli");
const http_1 = require("../http");
const util_2 = require("../util");
const wsRouter_1 = require("../wsRouter");
const errors_1 = require("./errors");
class CodeServerRouteWrapper {
    constructor() {
        this._wsRouterWrapper = (0, wsRouter_1.Router)();
        this.router = express.Router();
        //#region Route Handlers
        this.$root = async (req, res, next) => {
            const isAuthenticated = await (0, http_1.authenticated)(req);
            if (!isAuthenticated) {
                return (0, http_1.redirect)(req, res, "login", {
                    // req.baseUrl can be blank if already at the root.
                    to: req.baseUrl && req.baseUrl !== "/" ? req.baseUrl : undefined,
                });
            }
            next();
        };
        this.$proxyRequest = async (req, res, next) => {
            // We allow certain errors to propagate so that other routers may handle requests
            // outside VS Code
            const requestErrorHandler = (error) => {
                if (error instanceof Error && ["EntryNotFound", "FileNotFound", "HttpError"].includes(error.message)) {
                    next();
                }
                (0, errors_1.errorHandler)(error, req, res, next);
            };
            req.once("error", requestErrorHandler);
            this._codeServerMain.handleRequest(req, res);
        };
        this.$proxyWebsocket = async (req) => {
            this._codeServerMain.handleUpgrade(req, req.socket);
            req.socket.resume();
        };
        //#endregion
        /**
         * Fetches a code server instance asynchronously to avoid an initial memory overhead.
         */
        this.ensureCodeServerLoaded = async (req, _res, next) => {
            if (this._codeServerMain) {
                // Already loaded...
                return next();
            }
            if (constants_1.isDevMode) {
                // Is the development mode file watcher still busy?
                const compileStats = await (0, util_2.readCompilationStats)();
                if (!compileStats || !compileStats.lastCompiledAt) {
                    return next(new Error("VS Code may still be compiling..."));
                }
            }
            // Create the server...
            const { args } = req;
            /**
             * @file ../../../vendor/modules/code-oss-dev/src/vs/server/main.js
             */
            const createVSServer = await (0, util_2.loadAMDModule)("vs/server/remoteExtensionHostAgent", "createServer");
            try {
                this._codeServerMain = await createVSServer(null, await (0, cli_1.toVsCodeArgs)(args));
            }
            catch (createServerError) {
                (0, util_1.logError)(logger_1.logger, "CodeServerRouteWrapper", createServerError);
                return next(createServerError);
            }
            return next();
        };
        this.router.get("/", this.ensureCodeServerLoaded, this.$root);
        this.router.all("*", http_1.ensureAuthenticated, this.ensureCodeServerLoaded, this.$proxyRequest);
        this._wsRouterWrapper.ws("/", http_1.ensureAuthenticated, this.ensureCodeServerLoaded, this.$proxyWebsocket);
    }
    get wsRouter() {
        return this._wsRouterWrapper.router;
    }
    dispose() {
        this._codeServerMain?.dispose();
    }
}
exports.CodeServerRouteWrapper = CodeServerRouteWrapper;
//# sourceMappingURL=vscode.js.map